Package: ProduceR
Title: Concise and Efficient Tools for Everyday Statistical Production
Version: 1.1
Authors@R: c(person("Vincent", "Reduron", email = "vincent.reduron@laposte.net", role = c("cre", "aut")))
Description: A set of concise and efficient tools for statistical production. Can also be used for data management.
    In statistical production, you deal with complex data and need to control your process at each step of your work.
    Concise functions are very helpful, because you do not hesitate to use them.
    The following functions are included in the package.
    'dup'  checks duplicates.
    'miss' checks missing values.
    'tac' computes contingency table of all columns.
    'toc' compares two tables, spotting significant deviations.
    'chi2_find' compares columns within a data.frame, spotting related categories of (a more complex function).
Encoding: UTF-8
Imports: dplyr, rlang, tibble
RoxygenNote: 7.3.2
License: MIT + file LICENSE
NeedsCompilation: no
Packaged: 2026-01-16 16:13:06 UTC; vincent.reduron
Author: Vincent Reduron [cre, aut]
Maintainer: Vincent Reduron <vincent.reduron@laposte.net>
Repository: CRAN
Date/Publication: 2026-01-16 16:40:02 UTC
Built: R 4.6.0; ; 2026-04-25 20:45:05 UTC; unix

# Generated by roxygen2: do not edit by hand

export("%+%")
export(chi2_find)
export(coltypes)
export(dup)
export(get_recursion_depth)
export(is.Date)
export(is.POSIXct)
export(is.POSIXlt)
export(is.POSIXt)
export(miss)
export(tac)
export(toc)
export(toc_score)
import(dplyr)
importFrom(rlang,parse_expr)
importFrom(stats,complete.cases)
importFrom(stats,setNames)
importFrom(tibble,rownames_to_column)
importFrom(utils,View)
importFrom(utils,head)

The gCube System - the gCube messaging producer
------------------------------------------------------------

This work has been partially supported by the following European projects: DILIGENT (FP6-2003-IST-2), 
D4Science (FP7-INFRA-2007-1.2.2), D4Science-II (FP7-INFRA-2008-1.2.2), iMarine (FP7-INFRASTRUCTURES-2011-2), 
and EUBrazilOpenBio (FP7-ICT-2011-EU-Brazil).


Authors
-------

* Andrea Manzi (andrea.manzi@cern.ch), CERN, Geneva 
 
Version and Release Date
------------------------
v. 1.1.0 (07-09-12)
    
 
Description
-----------


Download information
--------------------

Source code is available from SVN:
${scm.url} 


Documentation 
-------------

https://gcube.wiki.gcube-system.org/gcube/index.php/Messaging_Infrastructure#Local_Producer

Licensing
---------

This software is licensed under the terms you may find in the file named "LICENSE" 
in this directory.

#' @title Missing: Generate a synthetic table of missing values for all columns of a data.frame
#'
#' @import dplyr
#' @importFrom tibble rownames_to_column
#' @importFrom stats setNames
#' @importFrom stats complete.cases
#' 
#' @description Get a synthetic table of missing values for all columns of a data.frame
#'
#' @param df data.frame: Input data.frame
#' @param values column: Variable (~weight) to measure the number of missing values (otherwise, count of rows)
#' @param view boolean: Display a glimpse of cases with NA values
#'
#' @return data.frame
#'
#' @examples miss(mtcars)  # Checking NA values for all columns of mtcars (none)
#'
#' @export
miss <- function(df, values = NULL, view = FALSE) {

  if (!is.null(groups(df))) df <- df %>% ungroup()
  
  # Table listing NAs per variable: raw
  if (is.null(values)) {
    stat <- df %>% summarise_all(~ sum(is.na(.)))
  } else {
    stat <- df %>% summarise(across(everything(), ~ sum(.data[[values]][is.na(.)], na.rm = TRUE)))
  }

  # Detailed view of missing values
  if (view == TRUE) {
    glimpse(df[!complete.cases(df), ])
  }

  # Table listing NAs per variable: final form
  # Note: To handle cases where df is a reference to a remote table and not an R table: added collect() instruction
  return(tibble::rownames_to_column(setNames(data.frame(t(stat %>% collect())), c("Missing")), var = "Variable"))
}

#' base_eu_2025
#'
#' Description.
#'
#' @format A data frame with 295 rows and 11 variables:
#' \describe{
#'   \item{ pays }{  character }
#'   \item{ region }{  character }
#'   \item{ annee }{  integer }
#'   \item{ Population }{  integer }
#'   \item{ PIB }{  numeric }
#'   \item{ PIB_habitant }{  numeric }
#'   \item{ Taux_Croissance }{  numeric }
#'   \item{ Taux_Chomage }{  numeric }
#'   \item{ Inflation }{  numeric }
#'   \item{ Dette_Publique }{  numeric }
#'   \item{ Balance_Commerciale }{  numeric }
#' }
#' @source Source
"base_eu_2025"

# VR voir comment il compte les key NA
#' @title Analysis of the cardinality of a key/identifier in a table
#'
#' @description Creates multiple result tables.
#' The term "n-plicate" is used to generalize the notion of duplicate: a n_plicate can be a duplicate, a triplicate, etc.
#'
#' @import dplyr
#' @importFrom utils View
#' @importFrom utils head
#' 
#' @param tab Either an R dataframe or a reference to a remote table ("remote table")
#' @param keyby (character vector) names of the column(s) considered as keys
#' @param count_what (character vector) defines what to count by key (by *keyby*).
#' 'rows' to count distinct rows, otherwise the name of the columns whose distinct values are to be counted
#' @param partition (character vector) names of the columns by which to break down the analysis
#' @param view automatic opening of generated tables
#' @param nb_xmpl number of duplicate examples displayed in table 
#'
#' @return A set of dataframes in the global environment.
#' * nup_r_tab: table of n-plicate counts
#' * nup_xmpl_dupl: table of examples of n-plicates 
#' * nup_xmpl_nakey: table of examples of NA keys (n-plicates with value 0)
#' * nup_r_tab_part: table of n-plicate counts broken down by the modalities of the `partition` columns
#'
#' @examples 
#' # Check if "name" is a unique key of the starwars table (yes !)
#' dup(dplyr::starwars, keyby = "name", view = FALSE)
#' 
#' # Check if "key" is a unique key of the basic table (no !)
#' basic <- data.frame("key"   = c("a", "b", "c", "d", NA, "a", "e", "f"), 
#'                     "value" = c(112, 117, 317,  NA,  0,  17, 117, 112))
#' dup(basic, keyby = "key", view = FALSE)
#' 
#' @export
dup <- function(tab, keyby = NULL, count_what = "rows", partition = NULL, view = TRUE, nb_xmpl = 51) {
  
  if(is.null(keyby)) keyby <- colnames(tab)
  
  # (I) Intermediate table with the same structure as the input table, but with an additional column named `cardinal` giving the number of n-plicates
  # -----------------------------------------------------------------------------------------------------------------
  
  # Alas, n_distinct({{count_what}}) only accepts a single column, I tried in vain with across(all_of(count_what))
  if(length(count_what) > 1){
  # Thus, going on with an intermediate column
    tab <- tab %>% 
      mutate(across(count_what), ~ as.character(.)) %>%
      mutate(concat_count_what = paste0(count_what))
  # Define the intermediate column as the new `count_what`
  count_what = "concat_count_what"
  }
  
  if (count_what != "rows") {
    nup_i_all <- tab %>%
      group_by(across(all_of(!!keyby))) %>%
      mutate(cardinal = n_distinct(!!sym(count_what))) %>%
      ungroup() 
  } else {
    nup_i_all <- tab %>%
      group_by(across(all_of(!!keyby))) %>%
      mutate(cardinal = n()) %>%
      ungroup()
  }

  # (II) Result tables
  # -----------------------------------------------------------------------------------------------------------------
  
  # Result 1: table of n-plicate counts (nup_r_tab = Results table of n_plicates)
  
    nup_r_tab <- nup_i_all %>%
      group_by(cardinal) %>%
      summarise(nb_lignes = n()) %>%
      mutate(nb_clefs = nb_lignes / cardinal) %>%
      arrange(cardinal) %>%
      collect()
    
    if (count_what != 'rows') nup_r_tab <- nup_r_tab %>% select(-nb_clefs)
    
    if (view) View(nup_r_tab)
    
  # Result 2: examples of n-plicates
    
    if (!(tally(nup_r_tab) == 1 && sum(nup_r_tab$cardinal) == 1)) # if there are duplicates
    {
      nup_xmpl_dupl <- nup_i_all %>%
        filter(cardinal > 1) %>%
        arrange(across(all_of(!!keyby))) %>%
        head(nb_xmpl) %>%
        collect()
      
      if (view) View(nup_xmpl_dupl)
    }
  
  # Result 3: examples of zero-plons (rows whose key is NA)
  
    nup_xmpl_nakey <- nup_i_all %>% filter(if_any(all_of(!!keyby), is.na))
    
    if (view) if(nrow(nup_xmpl_nakey) > 0) View(nup_xmpl_nakey)
  
  # Result 4: modalities of the partition column when there are n-plicates
  
    if (!is.null(partition)) {
      nup_r_tab_part <- nup_i_all %>%
        group_by(!!sym(partition), cardinal) %>%
        summarise(nb_clefs = n_distinct(!!sym(keyby)), nb_lignes = n()) %>%
        arrange(!!sym(partition), cardinal) %>%
        collect()
      
      if (view) if (!is.null(partition)) View(nup_r_tab_part) # VR issue: partition is not in nup_i_all
    }
  
  # list of outputs
    list_dup <- mget(ls(pattern = "^nup_r|^nup_xmpl"), envir = environment())
    return(list_dup)
}

utils::globalVariables(c(":=", "freq", "RAN_UNI_CNS", "cardinal", "chi2", "column", "expected_independence",
                         "indicator", "modality", "modality_margin_col_mod", "modality_margin_criterion", 
                         "nb_clefs", "score", "abscore", "col_typology",
                         "nb_lignes", "overall_margin", "iris"))

#' @title short for `paste0()`
#' @param a string
#' @param b string
#' @return string
#' @export
`%+%` <- function(a, b) paste0(a, b)


#' @title coltypes()
#' @description Create vector of df's column types. Similar to colnames(), but with column types instead of names.
#' @param df data.frame
#' @return vector
#' @export
coltypes <- function(df) sapply(df, typeof)


#' @title is.POSIXct
#' @description Returns TRUE or FALSE depending on whether its argument is of POSIXct type or not
#' @param x object
#' @return TRUE/FALSE
#' @export
is.POSIXct <- function(x) inherits(x, "POSIXct")

#' @title is.POSIXlt
#' @description Returns TRUE or FALSE depending on whether its argument is of POSIXlt type or not
#' @param x object
#' @return TRUE/FALSE
#' @export
is.POSIXlt <- function(x) inherits(x, "POSIXlt")

#' @title is.POSIXt
#' @description Returns TRUE or FALSE depending on whether its argument is of POSIXxt type or not
#' @param x object
#' @return TRUE/FALSE
#' @export
is.POSIXt  <- function(x) inherits(x, "POSIXt")

#' @title is.Date
#' @description Returns TRUE or FALSE depending on whether its argument is of Date type or not
#' @param x object
#' @return TRUE/FALSE
#' @export
is.Date    <- function(x) inherits(x, "Date")


#' @title get_recursion_depth
#' @description get recursion depth of a list
#' @param x : input list
#' @param depth : depth of x in another list (1 if x in a list. 2 if x is in a list of lists. Etc.)
#' @return integer
#' @export
get_recursion_depth <- function(x, depth = 0) {
  if (!is.list(x)) {
    if(depth == 0) stop("Your object in input of `get_recursion_depth()` is not a list.")
    return(depth)
  } else {
    max_depth <- depth
    for (item in x) {
      item_depth <- get_recursion_depth(item, depth + 1)
      max_depth <- max(max_depth, item_depth)
    }
    return(max_depth)
  }
}

#' @title Find modalities related to a criterion
#' 
#' @import dplyr
#' @importFrom rlang parse_expr
#' 
#' @param df data.frame
#' @param criterion character string: criterion that spots target rows
#' 
#' @return data.frame
#' 
#' @export
chi2_find <- function(df, criterion) {
  
  result <-
    
  # Cross table with `indicator` for all columns in the table
    bind_rows(
      df %>% filter(  !!rlang::parse_expr(criterion) ) %>% tac() %>% mutate(indicator = TRUE),
      df %>% filter(!(!!rlang::parse_expr(criterion))) %>% tac() %>% mutate(indicator = FALSE)
    ) %>%
    
  # Preparation: calculate margins
    
    # overall margin (each column cross has exactly this total)
    group_by(column) %>%
    mutate(overall_margin = sum(freq, na.rm = TRUE)) %>%
    ungroup() %>%
    
    # margin for indicator modalities (TRUE or FALSE)
    group_by(indicator) %>%
    mutate(modality_margin_criterion = sum(freq, na.rm = TRUE) / n_distinct(column)) %>%
    ungroup() %>%
    
    # margin for modalities of different columns
    group_by(column, modality) %>%
    mutate(modality_margin_col_mod = sum(freq, na.rm = TRUE)) %>%
    ungroup() %>%
    
  # Indicators of correlation between modalities 1 and 2
    mutate(
      expected_independence = modality_margin_criterion * modality_margin_col_mod / overall_margin,
      chi2 = round((freq - expected_independence) / expected_independence * (freq - expected_independence), 2),
      sign = ifelse(freq < expected_independence, '-', '+'),
      modality_among_criterion = round(freq / modality_margin_criterion, 2),
      criterion_among_modality = round(freq / modality_margin_col_mod  , 2),
      modality_among_whole     = round(modality_margin_col_mod   / overall_margin, 2),
      criterion_among_whole    = round(modality_margin_criterion / overall_margin, 2)
    ) %>%
    
    # Final table: the strongest correlations with TRUE are of interest, not with FALSE
    filter(indicator == TRUE, sign == '+') %>%
    arrange(desc(indicator), desc(chi2)) %>%
    select(-indicator, -sign, -col_typology, -freq, -modality_margin_criterion, -overall_margin, -expected_independence, -format) %>%
    mutate(criterion = criterion) %>%
    relocate(chi2, criterion, column, modality, modality_margin_col_mod) %>%
    rename(freq_modality = modality_margin_col_mod) 
}

#' @example stat <- chi2_find(iris, "Petal.Width == 1.0")

#' @title TAC-based Outlier Control (TOC)
#'
#' @description Generalized detection of outlier values in a database, based on contingency tables (tac)
#'
#' @import dplyr
#'
#' @param df1 Input data.frame (to compare with df2)
#' @param df2 Input data.frame (to compare with df1)
#' @param values Vector of columns that serve as measures (amounts, counts, etc.)
#' @param a Allowed absolute variation
#' @param r Allowed relative variation
#' @param sample_rate Sampling rate, if df is a remote table
#' @param num_but_discrete Numeric variables to be treated as discrete modal variables. If 'all', all numeric variables are treated as discrete modal variables.
#'
#' @return data.frame
#' @export
toc <- function(df1, df2, values = NULL, a = 10, r = 0.34, sample_rate = 0.01, num_but_discrete = 'NULL') {
  
  dfname1 <- deparse(substitute(df1))
  dfname2 <- deparse(substitute(df2))

  # This program only accepts a single quantity/amount variable
  if(length(values) > 1) stop("The input values vector has multiple elements, which is not accepted by this function.")

  # Both data.frames must have the same structure
  col1 <- colnames(df1)
  col2 <- colnames(df2)
  if(!identical(col1, col2)) warning("Different structure between the two tables. Comparison will be made on common columns.")
  common_cols <- intersect(col1, col2)
  df1 <- df1 %>% select(all_of(common_cols))
  df2 <- df2 %>% select(all_of(common_cols))

  # Generalized contingency tables for both data.frames
  tac1 <- tac(df1, values, sample_rate, num_but_discrete)
  tac2 <- tac(df2, values, sample_rate, num_but_discrete)

  # Names of the values on which detection is based
  if(is.null(values)) {
    value <- "freq"
    if(missing(a)) a <- pmin(ceiling(mean(nrow(df1), nrow(df2)) / 1e4), 10) # if the score is based on the number of observations (rather than the sum of a 'values' variable), scale it down to under 10 for small tables
  } else {
    value <- paste0("sum_", values[[1]])
  }
  value.x <- paste0(value, ".x")
  value.y <- paste0(value, ".y")

  # Outlier value detection
  tt <- full_join(tac1, tac2, by = c("column", "format", "modality"), na_matches = "na") %>% 
    mutate(across(c(value.x, value.y), ~ coalesce(., 0)))
  tt$score <- mapply(toc_score, tt[[value.x]], tt[[value.y]], a)
  tt$score <- ifelse(abs(tt$score) > r, tt$score, 0)
  tt$format <- NULL
  tt$abscore <- abs(tt$score)

  # Conclusion
  return(tt %>% arrange(desc(abscore), column) %>% select(all_of(c("column", "modality", value.x, value.y, "score"))) %>%
           rename(!!dfname1 := value.x, !!dfname2 := value.y))
}

#' @title Scoring significativity of difference between two values x and y
#' @description Difference score between x and y (0 = no significant difference, >0 = presence of significant difference)
#' @param x (num) First value to compare
#' @param y (num) Second value to compare
#' @param a (num) Absolute difference threshold below which all differences are considered normal
#' @return numeric
#' @examples toc_score(15, 1500, a = 500) # 1.91: significant difference
#' @examples toc_score(1432, 1501, a = 100) # 0: non-significant difference
#'
#' @export
toc_score <- function(x, y, a) {
  logistic <- function(x) 1 / (1 + exp(-x))
  x <- ifelse(is.na(x), 0, x)
  y <- ifelse(is.na(y), 0, y)
  if      (x + a < y)  { logistic(y/(x + a) - 1)}
  else if (x - a <= y & y <= x + a)  { 0 }
  else if (y < x - a)  {-logistic(x/(y + a) - 1)}
}

#' @title Computes a contingency table (tac) of all columns in a dataframe for control purposes
#'
#' @description Contingency table (tac) of all columns in a dataframe for control purposes
#'
#' @import dplyr
#'
#' @param df Input data.frame
#' @param values Vector of columns that serve as measures (amounts, counts, etc.)
#' @param sample_rate Sampling rate, if df is a remote table
#' @param num_but_discrete Vector of names of numeric columns with discrete modalities (not continuous)
#' @param strates Vector of column names by which to stratify the contingency tables
#'
#' @return data.frame
#'
#' @examples
#' tab <- tac(iris) # calculate column frequencies
#'
#' @export
tac <- function(df, values = NULL, sample_rate = 0.01, num_but_discrete = 'NULL', strates = NULL) {
  
  if(nrow(df) == 0) stop("The input table for the tac() function has no observations")

  # (1) Create an empty data.frame to store results
  tac_stock <- data.frame(modality = character(), freq = integer(), column = character(), format = character())

  # (2) Collect remote table, if df is a remote table instead of an R data.frame
  if (identical(class(df), c("tbl_Oracle", "tbl_dbi", "tbl_sql", "tbl_lazy", "tbl"))) {
    if(sample_rate != 1) {
      df <- df %>% mutate(RAN_UNI_CNS = sql("MOD(ROWNUM, 1000)") / 1000) %>% filter(RAN_UNI_CNS < sample_rate) %>% collect()
    } else {
      df <- df %>% collect()
    }
  }

  # (3) Loop through each column in the data.frame
  
  for (col_name in colnames(df)) {
    
    # Compute `modality` - which is a appropriate transformation of column values
    
      # Identifier column (> 85 distinct values) : keep filling in (Y/N)
      if(length(unique(df[[col_name]])) > 85 & !(col_name %in% num_but_discrete) & num_but_discrete[[1]] != 'all') {
        
        col_typology = 'identifier'
        df <- df %>% mutate(modality = ifelse(is.na(modality), "not filled in (NA)", "filled in"))
        
      # Date or datetime column : keep year 
      } else if(is.POSIXt(df[[col_name]]) | is.Date(df[[col_name]])) {
        
        col_typology = 'date'
        df <- df %>% mutate(modality := format(.data[[col_name]], "%Y"))

      # Numeric value : keep sign (positive, negative, zero or NA)
      } else if(is.numeric(df[[col_name]]) | is.double(df[[col_name]])) {
        
        col_typology <- 'quantitive'
        df <- df %>% mutate(modality := case_when(is.null(.data[[col_name]]) ~ "NULL", 
                                                  is.na(  .data[[col_name]]) ~ "NA", 
                                                  .data[[col_name]]  < 0 ~ "negative", 
                                                  .data[[col_name]] == 0 ~ "equal to 0", 
                                                  .data[[col_name]]  > 0 ~ "positive" 
        ))

      # Other cases such as character, boolean, etc, with < 85 distinct values : keep whole value (levels)
      } else {
        
        col_typology <- 'levels'
        df <- df %>% mutate(modality := as.character(.data[[col_name]]))
        
      }

    # Result data.frame (tac_...) for the column
    tac_column <- get_tac_column(df, col_name, values, strates) %>%
      mutate(col_typology = col_typology)

    # Iteration
    tac_stock <- bind_rows(tac_stock, tac_column)
  }

  # (4) Select & arrange: to manage column and row order
  return(tac_stock %>% arrange(column, modality) %>% relocate(column, format, col_typology, modality))
  
}

#' Contingency table for column `col_name` in data.frame `df`
#'
#' @param col_name string : name of column to which generate the contingency table
#' @inheritParams tac
get_tac_column <- function(df, col_name, values, strates) {
  
  # Compute number of rows
  if(is.null(values)) {
    return(df %>% group_by(across(all_of(c("modality", strates)))) %>%
             summarise(freq = n()) %>%
             mutate(column = col_name, format = typeof(df[[col_name]])) %>%
             collect())
  
  # Compute sum of values
  } else {
    return(df %>% group_by(across(all_of(c("modality", strates)))) %>%
             summarise(freq = n(), across(all_of(values), ~ round(sum(., na.rm = TRUE), 2), .names = "sum_{.col}")) %>%
             mutate(column = col_name, format = typeof(df[[col_name]])) %>%
             collect())
  }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tac.R
\name{tac}
\alias{tac}
\title{Computes a contingency table (tac) of all columns in a dataframe for control purposes}
\usage{
tac(
  df,
  values = NULL,
  sample_rate = 0.01,
  num_but_discrete = "NULL",
  strates = NULL
)
}
\arguments{
\item{df}{Input data.frame}

\item{values}{Vector of columns that serve as measures (amounts, counts, etc.)}

\item{sample_rate}{Sampling rate, if df is a remote table}

\item{num_but_discrete}{Vector of names of numeric columns with discrete modalities (not continuous)}

\item{strates}{Vector of column names by which to stratify the contingency tables}
}
\value{
data.frame
}
\description{
Contingency table (tac) of all columns in a dataframe for control purposes
}
\examples{
tab <- tac(iris) # calculate column frequencies

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{\%+\%}
\alias{\%+\%}
\title{short for `paste0()`}
\usage{
a \%+\% b
}
\arguments{
\item{a}{string}

\item{b}{string}
}
\value{
string
}
\description{
short for `paste0()`
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chi2_find.R
\name{chi2_find}
\alias{chi2_find}
\title{Find modalities related to a criterion}
\usage{
chi2_find(df, criterion)
}
\arguments{
\item{df}{data.frame}

\item{criterion}{character string: criterion that spots target rows}
}
\value{
data.frame
}
\description{
Find modalities related to a criterion
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_recursion_depth}
\alias{get_recursion_depth}
\title{get_recursion_depth}
\usage{
get_recursion_depth(x, depth = 0)
}
\arguments{
\item{x}{: input list}

\item{depth}{: depth of x in another list (1 if x in a list. 2 if x is in a list of lists. Etc.)}
}
\value{
integer
}
\description{
get recursion depth of a list
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/doc_base_eu_2024.R
\docType{data}
\name{base_eu_2024}
\alias{base_eu_2024}
\title{base_eu_2024}
\format{
A data frame with 232 rows and 11 variables:
\describe{
  \item{ pays }{  character }
  \item{ region }{  character }
  \item{ annee }{  integer }
  \item{ Population }{  integer }
  \item{ PIB }{  numeric }
  \item{ PIB_habitant }{  numeric }
  \item{ Taux_Croissance }{  numeric }
  \item{ Taux_Chomage }{  numeric }
  \item{ Inflation }{  numeric }
  \item{ Dette_Publique }{  numeric }
  \item{ Balance_Commerciale }{  numeric }
}
}
\source{
Source
}
\usage{
base_eu_2024
}
\description{
Description.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/toc.R
\name{toc_score}
\alias{toc_score}
\title{Scoring significativity of difference between two values x and y}
\usage{
toc_score(x, y, a)
}
\arguments{
\item{x}{(num) First value to compare}

\item{y}{(num) Second value to compare}

\item{a}{(num) Absolute difference threshold below which all differences are considered normal}
}
\value{
numeric
}
\description{
Difference score between x and y (0 = no significant difference, >0 = presence of significant difference)
}
\examples{
toc_score(15, 1500, a = 500) # 1.91: significant difference
toc_score(1432, 1501, a = 100) # 0: non-significant difference

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/toc.R
\name{toc}
\alias{toc}
\title{TAC-based Outlier Control (TOC)}
\usage{
toc(
  df1,
  df2,
  values = NULL,
  a = 10,
  r = 0.34,
  sample_rate = 0.01,
  num_but_discrete = "NULL"
)
}
\arguments{
\item{df1}{Input data.frame (to compare with df2)}

\item{df2}{Input data.frame (to compare with df1)}

\item{values}{Vector of columns that serve as measures (amounts, counts, etc.)}

\item{a}{Allowed absolute variation}

\item{r}{Allowed relative variation}

\item{sample_rate}{Sampling rate, if df is a remote table}

\item{num_but_discrete}{Numeric variables to be treated as discrete modal variables. If 'all', all numeric variables are treated as discrete modal variables.}
}
\value{
data.frame
}
\description{
Generalized detection of outlier values in a database, based on contingency tables (tac)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dup.R
\name{dup}
\alias{dup}
\title{Analysis of the cardinality of a key/identifier in a table}
\usage{
dup(
  tab,
  keyby = NULL,
  count_what = "rows",
  partition = NULL,
  view = TRUE,
  nb_xmpl = 51
)
}
\arguments{
\item{tab}{Either an R dataframe or a reference to a remote table ("remote table")}

\item{keyby}{(character vector) names of the column(s) considered as keys}

\item{count_what}{(character vector) defines what to count by key (by *keyby*).
'rows' to count distinct rows, otherwise the name of the columns whose distinct values are to be counted}

\item{partition}{(character vector) names of the columns by which to break down the analysis}

\item{view}{automatic opening of generated tables}

\item{nb_xmpl}{number of duplicate examples displayed in table}
}
\value{
A set of dataframes in the global environment.
* nup_r_tab: table of n-plicate counts
* nup_xmpl_dupl: table of examples of n-plicates 
* nup_xmpl_nakey: table of examples of NA keys (n-plicates with value 0)
* nup_r_tab_part: table of n-plicate counts broken down by the modalities of the `partition` columns
}
\description{
Creates multiple result tables.
The term "n-plicate" is used to generalize the notion of duplicate: a n_plicate can be a duplicate, a triplicate, etc.
}
\examples{
# Check if "name" is a unique key of the starwars table (yes !)
dup(dplyr::starwars, keyby = "name", view = FALSE)

# Check if "key" is a unique key of the basic table (no !)
basic <- data.frame("key"   = c("a", "b", "c", "d", NA, "a", "e", "f"), 
                    "value" = c(112, 117, 317,  NA,  0,  17, 117, 112))
dup(basic, keyby = "key", view = FALSE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tac.R
\name{get_tac_column}
\alias{get_tac_column}
\title{Contingency table for column `col_name` in data.frame `df`}
\usage{
get_tac_column(df, col_name, values, strates)
}
\arguments{
\item{df}{Input data.frame}

\item{col_name}{string : name of column to which generate the contingency table}

\item{values}{Vector of columns that serve as measures (amounts, counts, etc.)}

\item{strates}{Vector of column names by which to stratify the contingency tables}
}
\description{
Contingency table for column `col_name` in data.frame `df`
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{is.POSIXct}
\alias{is.POSIXct}
\title{is.POSIXct}
\usage{
is.POSIXct(x)
}
\arguments{
\item{x}{object}
}
\value{
TRUE/FALSE
}
\description{
Returns TRUE or FALSE depending on whether its argument is of POSIXct type or not
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{coltypes}
\alias{coltypes}
\title{coltypes()}
\usage{
coltypes(df)
}
\arguments{
\item{df}{data.frame}
}
\value{
vector
}
\description{
Create vector of df's column types. Similar to colnames(), but with column types instead of names.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/miss.R
\name{miss}
\alias{miss}
\title{Missing: Generate a synthetic table of missing values for all columns of a data.frame}
\usage{
miss(df, values = NULL, view = FALSE)
}
\arguments{
\item{df}{data.frame: Input data.frame}

\item{values}{column: Variable (~weight) to measure the number of missing values (otherwise, count of rows)}

\item{view}{boolean: Display a glimpse of cases with NA values}
}
\value{
data.frame
}
\description{
Get a synthetic table of missing values for all columns of a data.frame
}
\examples{
miss(mtcars)  # Checking NA values for all columns of mtcars (none)

}
